# Some data that is being used in the WorldNames module to generate data

# Python imports
import string

email_domains, letters, genders= [
    "gmail.com",
    "yahoo.com",
    "outlook.com",
    "hotmail.com",
    "aol.com",
    "icloud.com",
    "protonmail.com",
    "yandex.com",
    "mail.com",
    "zoho.com"
], [letter for letter in string.ascii_lowercase], ['Agender',
 'Androgyne',
 'Androgynous',
 'Bigender',
 'Cis',
 'Cis',
 'Female',
 'Cis',
 'Male',
 'Cis',
 'Man',
 'Cis',
 'Woman',
 'Cisgender',
 'Cisgender',
 'Female',
 'Cisgender',
 'Male',
 'Cisgender',
 'Man',
 'Cisgender',
 'Woman',
 'Female',
 'Female',
 'to',
 'Male',
 'FTM',
 'Gender',
 'Fluid',
 'Gender',
 'Identity',
 'Gender',
 'Nonconforming',
 'Gender',
 'Questioning',
 'Gender',
 'Variant',
 'Genderqueer',
 'Intersex',
 'Male',
 'Male',
 'to',
 'Female',
 'MTF',
 'Neither',
 'Neutrois',
 'Non-Binary',
 'Non-binary',
 'Not',
 'specified',
 'Other',
 'Pangender',
 'Trans',
 'Trans',
 'Female',
 'Trans',
 'Male',
 'Trans',
 'Man',
 'Trans',
 'Person',
 'Trans',
 'Woman',
 'Trans*',
 'Trans*',
 'Female',
 'Trans*',
 'Male',
 'Trans*',
 'Man',
 'Trans*',
 'Person',
 'Trans*',
 'Woman',
 'Transfeminine',
 'Transgender',
 'Transgender',
 'Female',
 'Transgender',
 'Male',
 'Transgender',
 'Man',
 'Transgender',
 'Person',
 'Transgender',
 'Woman',
 'Transmasculine',
 'Transsexual',
 'Transsexual',
 'Female',
 'Transsexual',
 'Male',
 'Transsexual',
 'Man',
 'Transsexual',
 'Person',
 'Transsexual',
 'Woman',
 'Two-Spirit']